@@ -73,16 +73,18 @@ specify_prior_bsvar_msh = R6::R6Class(
     #' Create a new prior specification PriorBSVARMSH.
     #' @param N a positive integer - the number of dependent variables in the model.
     #' @param p a positive integer - the autoregressive lag order of the SVAR model.
+    #' @param d a positive integer - the number of \code{exogenous} variables in the model.
     #' @param M an integer greater than 1 - the number of Markov process' heteroskedastic regimes.
     #' @param stationary an \code{N} logical vector - its element set to \code{FALSE} sets the prior mean for the autoregressive parameters of the \code{N}th equation to the white noise process, otherwise to random walk.
     #' @return A new prior specification PriorBSVARMSH.
-    initialize = function(N, p, M, stationary = rep(FALSE, N)){
+    initialize = function(N, p, d = 0, M, stationary = rep(FALSE, N)){
       stopifnot("Argument N must be a positive integer number." = N > 0 & N %% 1 == 0)
       stopifnot("Argument p must be a positive integer number." = p > 0 & p %% 1 == 0)
+      stopifnot("Argument d must be a non-negative integer number." = d >= 0 & d %% 1 == 0)
       stopifnot("Argument stationary must be a logical vector of length N." = length(stationary) == N & is.logical(stationary))
       stopifnot("Argument M must be an integer number greater than 1." = M > 1 & M %% 1 == 0)
       
-      super$initialize(N, p, stationary)
+      super$initialize(N, p, d, stationary)
       self$sigma_nu         = 3
       self$sigma_s          = 1
       self$PR_TR            = matrix(1, M, M)
@@ -171,20 +173,22 @@ specify_starting_values_bsvar_msh = R6::R6Class(
     #' @param p a positive integer - the autoregressive lag order of the SVAR model.
     #' @param M an integer greater than 1 - the number of Markov process' heteroskedastic regimes.
     #' @param T a positive integer - the the time series dimension of the dependent variable matrix \eqn{Y}.
+    #' @param d a positive integer - the number of \code{exogenous} variables in the model.
     #' @param finiteM a logical value - if true a stationary Markov switching model is estimated. Otherwise, a sparse Markov switching model is estimated in which \code{M=20} and the number of visited states is estimated.
     #' @return Starting values StartingValuesBSVAR-MS.
-    initialize = function(N, p, M, T, finiteM = TRUE){
+    initialize = function(N, p, M, T, d = 0, finiteM = TRUE){
       stopifnot("Argument N must be a positive integer number." = N > 0 & N %% 1 == 0)
       stopifnot("Argument p must be a positive integer number." = p > 0 & p %% 1 == 0)
       stopifnot("Argument M must be an integer number greater than 1." = M > 1 & M %% 1 == 0)
       stopifnot("Argument T must be a positive integer number." = T > 0 & T %% 1 == 0)
+      stopifnot("Argument d must be a non-negative integer number." = d >= 0 & d %% 1 == 0)
       stopifnot("Argument finiteM must be a logical value." = is.logical(finiteM) & length(finiteM) == 1)
       
       if (!finiteM) {
         M = 20
       }
       
-      super$initialize(N, p)
+      super$initialize(N, p, d)
       
       self$sigma2         = matrix(1, N, M)
       self$PR_TR          = diag(M)
@@ -286,6 +290,7 @@ specify_bsvar_msh = R6::R6Class(
     #' @param p a positive integer providing model's autoregressive lag order.
     #' @param M an integer greater than 1 - the number of Markov process' heteroskedastic regimes.
     #' @param B a logical \code{NxN} matrix containing value \code{TRUE} for the elements of the structural matrix \eqn{B} to be estimated and value \code{FALSE} for exclusion restrictions to be set to zero.
+    #' @param exogenous a \code{(T+p)xd} matrix of exogenous variables. 
     #' @param stationary an \code{N} logical vector - its element set to \code{FALSE} sets the prior mean for the autoregressive parameters of the \code{N}th equation to the white noise process, otherwise to random walk.
     #' @param finiteM a logical value - if true a stationary Markov switching model is estimated. Otherwise, a sparse Markov switching model is estimated in which \code{M=20} and the number of visited states is estimated.
     #' @return A new complete specification for the bsvar model with Markov Switching Heteroskedasticity, BSVARMSH.
@@ -294,6 +299,7 @@ specify_bsvar_msh = R6::R6Class(
     p = 1L,
     M,
     B,
+    exogenous = NULL,
     stationary = rep(FALSE, ncol(data)),
     finiteM = TRUE
     ) {
@@ -303,6 +309,10 @@ specify_bsvar_msh = R6::R6Class(
       TT            = nrow(data)
       T             = TT - self$p
       N             = ncol(data)
+      d             = 0
+      if (!is.null(exogenous)) {
+        d           = ncol(exogenous)
+      }
       
       if (!finiteM) {
         M = 20
@@ -317,10 +327,10 @@ specify_bsvar_msh = R6::R6Class(
       }
       stopifnot("Incorrectly specified argument B." = (is.matrix(B) & is.logical(B)) | (length(B) == 1 & is.na(B)))
       
-      self$data_matrices   = specify_data_matrices$new(data, p)
+      self$data_matrices   = specify_data_matrices$new(data, p, exogenous)
       self$identification  = specify_identification_bsvars$new(N, B)
-      self$prior           = specify_prior_bsvar_msh$new(N, p, M, stationary)
-      self$starting_values = specify_starting_values_bsvar_msh$new(N, self$p, M, T, finiteM)
+      self$prior           = specify_prior_bsvar_msh$new(N, p, d, M, stationary)
+      self$starting_values = specify_starting_values_bsvar_msh$new(N, self$p, M, T, d, finiteM)
     }, # END initialize
     
     #' @description
@@ -515,7 +525,7 @@ specify_posterior_bsvar_msh = R6::R6Class(
     #' data(us_fiscal_lsuw)
     #' 
     #' # specify the model and set seed
-    #' specification  = specify_bsvar$new(us_fiscal_lsuw, p = 4)
+    #' specification  = specify_bsvar_msh$new(us_fiscal_lsuw, p = 4, M = 2)
     #' set.seed(123)
     #' 
     #' # estimate the model
